-------------------------------
-- RogueRotationHelper | Created by TheoChao --
-- This file handles determining the proper rotation --
-------------------------------

function debugTime()
    local t=GetTime()
    if( math.floor((t-math.floor(t))*50)<=0) then
        return true;
    else
        return false;
    end
end

-- Returns the spell ID from spell name
function RogueRotationHelper:GetSpellNameById(spellId)

    if (spellId == nil) then
        return nil
    end
    
    local spellName, rank, _, _, _, _, _, _, _ = GetSpellInfo(spellId)
    return spellName
    
end

-- Create an Array of spells used within RogueRotationHelper's rotation algorithm.
RogueRotationHelper.Spells = {
    --finishers
    ["Slice and Dice"]      = (5171),
    ["Rupture"]             = (1943),
    ["Recuperate"]          = (73651),
    ["Eviscerate"]          = (2098),

    --assassination cds
    ["Vendetta"]            = (79140),
    ["Cold Blood"]          = (14177),

    --combat cds
    ["Killing Spree"]       = (51690),
    ["Adrenaline Rush"]     = (13750),

    --subtlety cds
    ["Preparation"]         = (14185),
    ["Shadowstep"]          = (36554),
    ["Premeditation"]       = (14183),
    ["Shadow Dance"]        = (51713),

    --random cds
    ["Berserking"]          = (26297),
    ["Blood Fury"]          = (33697),
    ["Every Man"]           = (59752),
    ["Arcane Torrent"]      = (25046),
    ["Racial"]              = (nil),
    ["Vanish"]              = (1856),

    --cp builders
    ["Ambush"]              = (8676),
    ["Mutilate"]            = (1329),
    ["Backstab"]            = (53),
    ["Garrote"]             = (703),
    ["Sinister Strike"]     = (1752),
    ["Revealing Strike"]    = (84617),
    ["Hemorrhage"]          = (89775),

    --buffs/debuffs
    ["Envenom"]             = (32645),
    ["Overkill"]            = (58426),
    ["Deadly Poison"]       = (2818),
    ["Blade Flurry"]        = (13877),
    ["Find Weakness"]       = (91023),
    ["Master of Subtlety"]  = (31223),
    ["Honor Among Thieves"] = (51698),
    
    --bleed debuffs
    ["Blood Frenzy"]        = (30070),
    ["Gore"]                = (35290),
    ["Stampede"]            = (57386),
    ["Tendon Rip"]          = (50271),
    ["HemorrhageBleed"]     = (16511),
    ["Mangle"]              = (33876),

    --armor debuffs
    ["Expose Armor"]        = (8647),
    ["Faerie Fire"]         = (91565),
    ["Sunder Armor"]        = (58567),
    ["Tear Armor"]          = (95467),
    ["Corrosive Spit"]      = (95466),
    
    --other spells
    ["Kick"]                = (1766),
    ["Feint"]               = (1966),
    ["Tricks of the Trade"] = (57934),
        
}



function RogueRotationHelper:CalculateRotation()

    local energy = UnitPower("player") + self.db.profile.EnergyOffset
    local cp = GetComboPoints("player")

    local spell = ""
    local nextspell = ""
    
    local tott = 0
    local feint = 0
    local talent_tree = RogueRotationHelper.db.profile.TalentTree
    
    local soundfile = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\pingit.mp3"

    -- get the time left for various buffs/debuffs
    -- Feint
    local start, duration, enabled = GetSpellCooldown(self.Spells["Feint"]);
    if duration ~= nil then 
        feint = duration + start - GetTime()
    end
        
     -- Tricks
    local start, duration, enabled = GetSpellCooldown(self.Spells["Tricks of the Trade"]);
    if duration ~= nil then 
        tott = duration + start - GetTime()
    end

    
    -- first determine if the talent tree is assination or combat
    if talent_tree == 1 then
        spell, nextspell, extraWait = self:GetNextAssassinationSpell(energy, cp)
    -- in combat spec
    elseif talent_tree == 2 then
        spell, nextspell, _ = self:GetNextCombatSpell(energy, cp)
    -- in combat spec
    elseif talent_tree == 3 then
        spell, nextspell, _ = self:GetNextSubtletySpell(energy, cp)
    end
    
    if spell == "" then
        if self.db.profile.SuggestTricks and tott <= 0 then
            spell = self.Spells["Tricks of the Trade"];
        elseif self.db.profile.SuggestFeint and feint <= 0 then
            spell = self.Spells["Feint"];
        else
        end
    end

    if self.db.profile.SpellSounds[RogueRotationHelper:GetSpellNameById(spell)] ~= nil then
        soundfile = self.db.profile.SpellSounds[RogueRotationHelper:GetSpellNameById(spell)]
    end
    
    self.displayFrame_current.Texture:SetTexture(GetSpellTexture(spell))
    self.displayFrame_next.Texture:SetTexture(GetSpellTexture(nextspell))
    
    -- global cooldowns
    if (spell ~= "") and (spell ~= nil) then
        local start, dur = GetSpellCooldown(spell)
        globalcd = dur + start - GetTime()
        if globalcd >= .25 then
            -- Display global cooldowns
            self.displayFrame_current.Frame:SetCooldown(start, dur)
            
            -- Doesn't display anything as of yet, may be used in future with better queueing algorithm
            if(nextspell ~= "") and (nextspell ~= nil) then
                self.displayFrame_next.Frame:SetCooldown(start, dur)
            end
            self.db.profile.lastSpell = ""
        else
            if(self.db.profile.lastSpell ~= spell) then
                if self.db.profile.SoundEnabled then
                    PlaySoundFile(soundfile, "MASTER")
                end
                self.db.profile.lastSpell = spell
            end
        end
    end
    -- handle the extra wait timer bar
    if( extraWait ~= nil and extraWait > 0 and nextspell ~= "" and nextspell ~= nil) then
        self.displayFrame_current.Texture:SetTexture(GetSpellTexture(nextspell))
        self.displayFrame_current.Frame:SetCooldown(GetTime(), extraWait)
        self.displayFrame_current.FontString:SetText(string.format("%.1f", extraWait))
    else
        --self.displayFrame_current.Frame:SetCooldown(time(), 0)
        self.displayFrame_current.FontString:SetText("")
    end
end


function RogueRotationHelper:GetNextAssassinationSpell(energy, cp)
    local murderousIntent = 0
    
    if(self.db.profile.SuggestBackstab == true) and (UnitHealth("target") <= UnitHealthMax("target") * 0.35) then
        murderousIntent = 1
    end

    local spell = ""
    local nextspell = ""
    
    local garrote = 0
    local slice_and_dice = 0
    local rupture = 0
    local vendetta = 0
    local ruptureLevel = 0
    local deadly_poison_stack_count = 0
    local deadly_poison = 0
    local vanish = 0
    local overkill = 0
    local coldblood = 0
    local envenom = 0
    local extraWait = 0;

    -- get the time left for various debuffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitDebuff("target", i)
        if not icon then break end
        
        -- Rupture
        if(spellid==self.Spells["Rupture"] and unitCaster == "player")then
            rupture = expirationTime - GetTime()
        end
        
        -- Vendetta
        if spellid==self.Spells["Vendetta"] and unitCaster == "player" then
            vendetta = expirationTime - GetTime()
        end
        
        -- Deadly Poison
        if spellid==self.Spells["Deadly Poison"] and unitCaster == "player" then
            deadly_poison_stack_count = count
            deadly_poison = expirationTime - GetTime()
        end

        i = i + 1
    end
    
    -- get the time left for various buffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitBuff("player", i)
        if not icon then break end
        
        -- Slice and Dice
        if spellid==self.Spells["Slice and Dice"] then
            slice_and_dice = expirationTime - GetTime()
        end
        
        -- Overkill
        if spellid==self.Spells["Overkill"] then
            overkill = expirationTime - GetTime()
        end
        
        -- Envenom
        if spellid==self.Spells["Envenom"] then
            envenom = expirationTime - GetTime()
        end
        
        i = i + 1
    end
    
    -- Cold Blood
    local start, duration, enabled = GetSpellCooldown(self.Spells["Cold Blood"]);
    if duration ~= nil then 
        coldblood = duration + start - GetTime()
    end


    

    --------------------------------------------------
    -- start rotation
    --------------------------------------------------
    
    -- Stealth active
    if UnitAura("player", "Stealth") ~= nil then
        -- Open with Garrote
        if (energy >= 50) then
            spell = self.Spells["Garrote"]
        else
            nextspell = self.Spells["Garrote"];
        end
   -- Initial rupture
    elseif (cp >= 1) and (rupture == 0.0) then
        -- Check required energy
        if (energy >= 25) then
            spell = self.Spells["Rupture"];
        else
            nextspell = self.Spells["Rupture"];
        end
        
    -- Slice and Dice 
    elseif (cp >= 1) and (slice_and_dice == 0.0) then
        -- Check required energy
        if (energy >= 25) then
            spell = self.Spells["Slice and Dice"];
        else
            nextspell = self.Spells["Slice and Dice"];
        end
        
    -- Finish to refresh Slice and Dice if low on timer
    elseif (cp >= 1) and (deadly_poison_stack_count >= 1) and (slice_and_dice <= self.db.profile.SliceTimer and slice_and_dice > 0.25) then
        -- Check required energy
        if (energy >= 35) then
            spell = self.Spells["Envenom"];
        else
            nextspell = self.Spells["Envenom"];
        end
           
    -- Rupture if low on timer (but don't cap out on energy
    elseif (rupture <= self.db.profile.RuptureTimer) and (cp >= 1 ) and (energy < 105 or energy >= 120) then 
        extraWait = rupture;
        -- Queue if not quite finished, check required energy, make sure that able to refresh (or it's close to expiring) and don't go too early
        if (energy >= 25 and (cp >= ruptureLevel or rupture <.2) and rupture < 1.25) then
            spell = self.Spells["Rupture"];
        else
            -- if there's some time and lower than the rupture point - build more cp
            if (rupture>1 and cp < ruptureLevel and energy>60) then
                if(murderousIntent == 1 and cp < 5) then
                    spell = self.Spells["Backstab"];
                elseif(murderousIntent == 0 and cp < 4) then
                    spell = self.Spells["Mutilate"];
                else
                    nextspell = self.Spells["Rupture"]
                end
            else
                nextspell = self.Spells["Rupture"];
            end
        end
    
     -- Cold Blood
    elseif (coldblood <= 0 and ((cp >= 5 and vendetta > 0) or (cp >= 4 and vendetta > 0 and vendetta < 10))) then
        spell = self.Spells["Cold Blood"];
    
    -- Suggest Envenom (in a single case to account for murderous intent taking to 5 cp)
    elseif ( cp >= 4 + murderousIntent) then
        extraWait = envenom;
        --make sure that energy is pooled, there's dp up, and envenom isn't already on (unless maxing out on energy)
        if ( energy >= 55 and deadly_poison_stack_count == 5 and (envenom == 0 or energy >= 105)) then
            spell = self.Spells["Envenom"];
        else
            -- or if there's no stacks of dp, assume it's not on and eviscerate
            if (deadly_poison_stack_count == 0) then
                    if (energy >= 35) then
                        spell = self.Spells["Eviscerate"];
                    else
                        nextspell = self.Spells["Eviscerate"];
                    end
            else
                nextspell = self.Spells["Envenom"];
            end
        end
        
    -- build up combo points
    else
        if(murderousIntent == 1 ) then
            if (energy >= 60) then
                spell = self.Spells["Backstab"];
            else
                nextspell = self.Spells["Backstab"];
            end
        
        else
            if (energy >= 55) then
                spell = self.Spells["Mutilate"];
            else
                nextspell = self.Spells["Mutilate"];
            end
            
        end
    end
    return spell,nextspell,extraWait
end

function RogueRotationHelper:GetNextCombatSpell(energy, cp)
  
    local spell = ""
    local nextspell = ""
    
    local slice_and_dice = 0
    local rupture = 0
    local bleed = 0
    local armor = 0 
    local rev_strike = 0
    
    -- get the time left for various debuffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitDebuff("target", i)
        if not icon then break end
        --print(name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid);
        
        -- Rupture
        if(spellid==self.Spells["Rupture"] and unitCaster == "player")then
            rupture = expirationTime - GetTime()
        end
        
        -- Revealing Strike 
        if spellid==self.Spells["Revealing Strike"] and unitCaster == "player" then
            rev_strike = expirationTime - GetTime()
        end
        
        -- Bleed
        foreach({"Blood Frenzy","Gore","Stampede","Tendon Rip","HemorrhageBleed","Mangle"}, 
            function(k,v)
                if spellid == self.Spells[v] then
                    bleed=1
                end 
            end)

        -- Armor
        foreach({"Expose Armor","Faerie Fire","Sunder Armor","Tear Armor","Corrosive Spit"}, 
            function(k,v)
                if spellid == self.Spells[v] then
                    armor=1
                end 
            end)

        i = i + 1
    end
    
    -- get the time left for various buffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitBuff("player", i)
        if not icon then break end
        
        -- Slice and Dice
        if spellid==self.Spells["Slice and Dice"] then
            slice_and_dice = expirationTime - GetTime()
        end
    
        i = i + 1
    end
    
    --------------------------------------------------
    -- start rotation
    --------------------------------------------------
    
    -- Stealth active
    if UnitAura("player", "Stealth") ~= nil then
        -- Open with Garrote
        if (energy >= 50) then
            spell = self.Spells["Garrote"]
        else
            nextspell = self.Spells["Garrote"];
        end

    -- Slice and Dice if running low
    elseif (cp >= 1 ) and (slice_and_dice <= self.db.profile.SliceTimer) and rev_strike <= 0 then
        -- Check required energy
        if (energy >= 25) then
            spell = self.Spells["Slice and Dice"];
        else
            nextspell = self.Spells["Slice and Dice"];
        end
        
    -- armor if no armor buff
    elseif ( cp>=self.db.profile.SuggestArmorCP and armor<=0 ) then 
        -- Queue if not quite finished, check required energy
        if (energy >= 25) then
            spell = self.Spells["Expose Armor"];
        else
            nextspell = self.Spells["Expose Armor"];
        end
    -- Rupture if low on timer/out and bleed debuff is active
    elseif ( cp>=5 and bleed>0 and rupture <= self.db.profile.RuptureTimer and self.db.profile.SuggestRupture) then 
        -- Queue if not quite finished, check required energy
        if (energy >= 25) then
            spell = self.Spells["Rupture"];
        else
            nextspell = self.Spells["Rupture"];
        end
        
    -- Suggest Eviscerate
    elseif (cp >= 5) then
        -- Check required energy
        if (energy >= 35) then
            spell = self.Spells["Eviscerate"];
        else
            nextspell = self.Spells["Eviscerate"];
        end
        
    -- build cp
    else
        -- if at 4, rs for the single point
        if( cp >= 4  and rev_strike <= 0 ) then
            if (energy >= 40) then
                spell = self.Spells["Revealing Strike"];
            else
                nextspell = self.Spells["Revealing Strike"];
            end
        
        -- otherwise get 1 or 2 with ss
        else
            if (energy >= 39) then
                spell = self.Spells["Sinister Strike"];
            else
                nextspell = self.Spells["Sinister Strike"];
            end
        end
    end
    
     -- check for blade flurry
    if( UnitBuff("player", GetSpellInfo(self.Spells["Blade Flurry"])) ) then
        self.displayFrame_current.Border:SetTexture(1,0,0,1)
    else
        self.displayFrame_current.Border:SetTexture(0,0,0,0)
    end
    return spell,nextspell
end


function RogueRotationHelper:GetNextSubtletySpell(energy, cp)
    local spell = ""
    local nextspell = ""
    
    local slice_and_dice = 0
    local recuperate = 0
    local rupture = 0
    local vanish = 0
    local prep = 0
    local premed = 0
    local shadowstep = 0
    local stealthactive = 0
    local hemorrhage = 0
    local bleed = 0
    local master_of_subtlety = 0

    -- get the time left for various debuffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitDebuff("target", i)
        if not icon then break end
        
        -- Rupture
        if(spellid==self.Spells["Rupture"] and unitCaster == "player")then
            rupture = expirationTime - GetTime()
        end

        -- Hemorrhage
        if spellid==self.Spells["Hemorrhage"] and unitCaster == "player" then
            hemorrhage = expirationTime - GetTime()
        end
        
        -- Bleed
        foreach({"Blood Frenzy","Gore","Stampede","Tendon Rip","HemorrhageBleed","Mangle"}, 
            function(k,v)
                if spellid == self.Spells[v] then
                    bleed=1
                end 
            end)
            
        i = i + 1
    end
    
    -- get the time left for various buffs
    local i = 1
    while true do
        local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable,shouldConsolidate ,spellid = UnitBuff("player", i)
        if not icon then break end
        
        -- Slice and Dice
        if spellid==self.Spells["Slice and Dice"] then
            slice_and_dice = expirationTime - GetTime()
        end
    
        -- Recuperate 
        if spellid==self.Spells["Recuperate"] then
            recuperate = expirationTime - GetTime()
        end
        
        i = i + 1
    end

    -- Vanish
    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable = UnitBuff("player", self.Spells["Vanish"])
    if name ~= nil then
        vanish = expirationTime - GetTime()
    end

    -- Preparation
    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable = UnitBuff("player", self.Spells["Preparation"])
    if name ~= nil then
        prep = expirationTime - GetTime()
    end

    -- Premeditation
    local start, duration, enabled = GetSpellCooldown(self.Spells["Premeditation"]);
    if duration ~= nil then 
        premed = duration + start - GetTime()
    end

    -- Shadowstep
    local start, duration, enabled = GetSpellCooldown(self.Spells["Shadowstep"]);
    if duration ~= nil then 
        shadowstep = duration + start - GetTime()
    end
    
    -- Shadow Dance
    if UnitAura("player", "Shadow Dance") ~= nil then
        stealthactive = 1
    end
    
    if UnitAura("player", "Stealth") ~= nil then
        stealthactive = 1
    end

    
    -- Master of Subtlety
    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable = UnitBuff("player", self.Spells["Master of Subtlety"])
    if name ~= nil then
        master_of_subtlety = expirationTime - GetTime()
    end
    
    --------------------------------------------------
    -- start rotation
    --------------------------------------------------
    
    -- Finisher section
    if (cp >= 4) then
        if( cp == 4 ) then
            --wait for honor among thieves to proc a cp
            spell = self.Spells["Honor Among Thieves"];
        elseif (rupture <= 0 and master_of_subtlety > 0) then
            if (energy >= 25) then
                spell = self.Spells["Rupture"];
            else
                nextspell = self.Spells["Rupture"];
            end
        elseif (slice_and_dice <= 0) then
            if (energy >= 25) then
                spell = self.Spells["Slice and Dice"];
            else
                nextspell = self.Spells["Slice and Dice"];
            end
        elseif (rupture <= 0 and master_of_subtlety <= 0) then
            if (energy >= 25) then
                spell = self.Spells["Rupture"];
            else
                nextspell = self.Spells["Rupture"];
            end
        elseif (recuperate <= 0) then
            if (energy >= 30) then
                spell = self.Spells["Recuperate"];
            else
                nextspell = self.Spells["Recuperate"];
            end
        else
            if (energy >= 35) then
                spell = self.Spells["Eviscerate"];
            else
                nextspell = self.Spells["Eviscerate"];
            end
        end
    -- build up combo points
    else
        -- if Stealth active check for cooldowns
        if ( stealthactive > 0) then
            if (cp<=3 and premed<=0) then
                spell = self.Spells["Premeditation"]
            elseif (shadowstep<=0) then
                spell = self.Spells["Shadowstep"]
            else
            -- Ambush
            if (energy >= 40) then
                spell = self.Spells["Ambush"]
            else
                nextspell = self.Spells["Ambush"];
                end
            end
            
        elseif(hemorrhage <= 0) then 
            if (energy >= 29) then
                spell = self.Spells["Hemorrhage"];
            else
                nextspell = self.Spells["Hemorrhage"];
            end
            
        elseif(self.db.profile.SuggestBackstab == true) then
            if (energy >= 40) then
                spell = self.Spells["Backstab"];
            else
                nextspell = self.Spells["Backstab"];
            end
        
        else
            if (energy >= 29) then
                spell = self.Spells["Hemorrhage"];
            else
                nextspell = self.Spells["Hemorrhage"];
            end
            
        end
    end
    return spell,nextspell
end